import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { UpdateConfigurationCommand } from '../../commands/update-configuration.command';
import { UpdateConfigurationRequest } from '../dtos/update-configuration.request';
import { Configuration } from '../entities/configuration';

@CommandHandler(UpdateConfigurationCommand)
export class UpdateConfigurationUseCase {
  constructor(
    private readonly _repository: ConfigurationRepository,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: UpdateConfigurationCommand): Promise<Configuration> {
    const entity = this._mapper.map(
      command.updateConfigurationRequest,
      UpdateConfigurationRequest,
      Configuration,
    );

    try {
      const configuration = await this._repository.update(
        command.updateConfigurationRequest.uuid,
        entity,
      );
      return configuration;
    } catch (error) {
      throw error;
    }
  }
}
