import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { UpdateConfigurationCommand } from '../../commands/update-configuration.command';
import { UpdateConfigurationRequest } from '../../domain/dtos/update-configuration.request';
import { Configuration } from '../../domain/entities/configuration';
import { UpdateConfigurationUseCase } from '../../domain/usecases/update-configuration.usecase';
import { ConfigurationProfile } from '../../mappers/configuration.profile';

const originalConfiguration: Configuration = new Configuration();
originalConfiguration.uuid = 'bb281075-1b98-4456-89d6-c643d3044a91';
originalConfiguration.key = 'key1';

const updateConfigurationRequest: UpdateConfigurationRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  key: 'Dane',
};

const updateConfigurationCommand: UpdateConfigurationCommand =
  new UpdateConfigurationCommand(updateConfigurationRequest);

const mockConfigurationRepository = {
  update: jest
    .fn()
    .mockImplementationOnce((uuid: string, params: any) => {
      originalConfiguration.key = params.key;

      return Promise.resolve(originalConfiguration);
    })
    .mockImplementation(() => {
      throw new Error('Error');
    }),
};

describe('UpdateConfigurationUseCase', () => {
  let updateConfigurationUseCase: UpdateConfigurationUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],

      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        UpdateConfigurationUseCase,
        ConfigurationProfile,
      ],
    }).compile();

    updateConfigurationUseCase = module.get<UpdateConfigurationUseCase>(
      UpdateConfigurationUseCase,
    );
  });

  it('should be defined', () => {
    expect(updateConfigurationUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should update a configuration', async () => {
      const updatedConfiguration: Configuration =
        await updateConfigurationUseCase.execute(updateConfigurationCommand);

      expect(updatedConfiguration.key).toBe(updateConfigurationRequest.key);
    });
    it('should throw an error if configuration does not exist', async () => {
      await expect(
        updateConfigurationUseCase.execute(updateConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
